/*
m_segk - output a segment of variable characteristic between two points

DESCRIPTION
m_segk outputs a segment between two points. The segment characteristic can
vary between three types, e.g. logarithmic, linear and exponential. The segment
shapes are loaded from user supplied function tables.

SYNTAX
kres, kfin m_segk kstart, ktime, kend, ktype, ktrig

INITIALIZATION

PERFORMANCE
kres - The current value of the segment
kfin - kfin is 1, when the segment is completed (when the end value after the
	specified time has been reached.
kstart - Start value of the segment
ktime - Time it takes to move from kstart to kend
kend - End value of the segment
ktype - Type of the segment. The type is variable between the three supplied
	shapes. When ktype == 0, use ifn1, when ktype == 1, use ifn2 and when
	ktype == 2, use ifn3. Values in between will blend between the supplied
	function tables using tabmorph
ktrig - Trigger for the segment. Start the segment if trigger == 1 .

CREDITS
Author: Jeanette C., heavily based on Joachim Heintz's UDO "Linek" found
on http://github.com/csudo/csudo/blob/master/lines/Linek.csd
Thank you Joachim!
*/

opcode m_segk, kk, kkkkk
	kstart, ktime, kend, ktype, ktrig xin

	; Create required ftables
	ifn1 ftgenonce 0, 0, 4096, -23, "log.ftable"
	ifn2 ftgenonce 0, 0, 4096, 7, 0, 4096, 1
	ifn3 ftgenonce 0, 0, 4096, -23, "exp.ftable"

	kstat init 0 ; internal state
	kcount init 0 ; step counter
	kperiod = round(kr * ktime) ; periods it takes to complete the line
	ilen tableng ifn2 ; Length of the table
	ktimek timek ; current time, later compared to start time for progress
	if ktrig == 1 then ; Memorise current time
		if ktime == 0 then ; only output the end value
			kval = kend
			kfin = 1
			kstat = 2
			kgoto Output
		else ; We have a real segment of time >0
			kstarttime = ktimek
			kstat = 1
			krange = abs(kend - kstart) ; Scaling factor for the control output
			if kstart < kend then
				koffset = kstart
			elseif kstart > kend then
				koffset = kend
			endif
		endif
	endif

	if kstat == 0 then ; Nothing has happened yet
		kval = kstart
	elseif kstat == 1 then ; we're running
		if kstart != kend then ; only calculate everything, if we don't have a straight line
			klowtype = floor(ktype) ; lower table number for the morph
			kweight = ktype - klowtype ; Fractional part of the type for weighting
			if kstart <= kend then ; read the table forwards
				kindex = (ktimek - kstarttime) / kperiod * (ilen-1) ; current index into the table(s)
			else ; read the table backwards
				kindex = (ilen-1) - ((ktimek - kstarttime) / kperiod * (ilen-1))
			endif
			kval tabmorph kindex, kweight, klowtype, (klowtype + 1), ifn1, ifn2, ifn3
			kval = (kval * krange) + koffset ; scale the table output to the actual
					; output range
		else
			kval = kstart
		endif
		if ktimek == (kstarttime + kperiod) then ; Time is up
			kfin = 1
			kstat = 2
		else ; We're not done yet
			kfin = 0
		endif
	elseif kstat == 2 then ; The line is finished
		kval = kval
		kfin = 0
	endif

Output:
	xout kval, kfin
endop
